#ifndef ACTOR_H
#define ACTOR_H

#include <d3d9.h>
#include <d3dx9.h>
#include <stdlib.h>
#include <Math.h>
#include "Weapon.h"
#include "Vector.h"
#include "iostream"
using namespace std;

#define STUPIDHEALTH 10	//the amount of health each enemy type will have
#define SMARTHEALTH  20
#define FLYERHEALTH  20
#define STUPIDDAMAGE 10 //the amount of damage each enemy type does
#define SMARTDAMAGE  20
#define FLYERDAMAGE  20

struct HealingPoint
{
	int xPos;
	int yPos;
	int zPos;
	int size;
	HealingPoint()
	{
	}
	HealingPoint(int x, int y, int z, int s)
	{
		xPos = x;
		yPos = y;
		zPos = z;
		size = s;
	}
};

enum EnemyType
{
	groundStupid,	//just moves towards the player, nothing special
	groundSmart,	//learning ai, has all the special functions
	flyer,			//same as groundsmart, except it flys
};

enum EnemyAction
{
	kamikaze,	//watch a WWII movie
	survivor,	//runs back to heal when wounded
	groupie,	//stays in groups
	convict,	//tries to attack from behind
	dodger,		//runs side to side in an eratic manner
	jump,		//jumps
	diveBomb,	//flyers only, swoops down to attack and then back up
	nothing		//in case its not an enemy
};

class ActorTemplate//See actor basicly simple information on an actor
{
	public:
	ActorTemplate(Vector size,float speed,float max_health,string name,int flags,Weapon *rweapon,Weapon *lweapon,int ai_level,string class_name,string trigger_message,float damage,float radius);
	Vector		angle;
	Vector		size;
	Vector		camera_view_ofs;
	float		speed;
	float		health;
	float		max_health;
	string		name;
	int			flags;
	Weapon		*rweapon;
	Weapon		*lweapon;

	int			MDLNUMBER;
	int			ai_level;
	string		class_name;
	string		trigger_message;
   	float		damage;
   	float		radius;
};

class Actor
{
	public:
	~Actor();
	Actor();
	Actor(Actor &a);
	Actor(ActorTemplate &at);

	Vector			origin;
	Vector			angle;
	Vector			size;
	Vector			camera_view_ofs;// Camera position relative to player
	float			speed;
	Actor			*owner;
	Actor			*other;
	Actor			*enemy;
	Actor			*next;

	int			MDLNUMBER;
	// Player stuff
	float		health;
	float		max_health;// players maximum health is stored here
	int			frags;
	string		name;
	int			team;
	int			exp;
	int			level;
	int			featPoints;
	int			flags;//basicly a list of 32 bool propertys
	Weapon		*rweapon;//Rweapon gets an accuracy bonus
	Weapon		*lweapon;//Lweapon is the optional alt weapon requires a special feat

	// Monster stuff
	int				ai_level;
	string			class_name;
	float			next_think;
	Vector			goal_point;
	HealingPoint	hPoint;
	float			*percentFrame;
	float			*playerHealth;
	int				attackArea;		//when a dodger is within this range it will go straight to the player
	int				attackRange;	//the range of the enemy attack
	int				attackCount;	//enemy attacks again when count hits cap
	int				attackCap;		//enemy attacks again when count hits cap
	int				dodgeOffset;	//how much the dodging enemy will go side to side
	int				dodgeCount;		//increments, enemy changes direction when it hits the cap
	int				dodgeCap;		//how often the enemy will dodge side to side
	int				jumpHeight;		//how high the enemy will jump
	int				jumpCount;		//increments, enemy jumps when it hits the cap
	int				jumpCap;		//how often the enemy will jump
	int				diveBombHeight;	//how high the flyer will be before dive bombing
	float			attackDamage;	//how much damage the enemy does
	EnemyType		enemyType;
	EnemyAction		currentAction;
	float			kamikazeRange;	//the range of probability the action will be called (out of 10)
	float			survivorRange;	//same for all ranges...
	float			groupieRange;	//...
	float			convictRange;	//...
	float			dodgerRange;	//...
	float			jumpRange;		//...
	float			diveBombRange;	//...till here
	int				actionAmount;
	bool			damageDone;		//if the enemy has successfully attacked the player
	bool			atHealPoint;
	HealingPoint	*healPoint;		//pointer to where the healing point is
	int				healCount;
	int				healCap;
	int				maxHealth;
	Vector			playerPos;
	int				jumpDirection;
	int				jumpTime;
	int				jumpTimeLimit;
	float			lastZ;
	int				jumpOffset;
	bool			jumping;
	int				jumpSpeed;
	int				jumpSpeedCount;

	//Item stuff
	string		trigger_message;// trigger messages
	float		dam_mod;// 0 = no damage 1 = normal damage 2 = float damage
	float		move_mod;// 0 = no movment 1 = normal movment 2 = float movment
	float		reload_mod;// 0 = Instant reload 1 = normal reload 2 = float reload 
	float		fire_mod;// 0 = no shooting 1 = normal shooting 2 = float shooting
	float		regen_mod;// 0 = no regeneration 1 = 1 hp/tic 2 = 2 hp/tic(FIXME tic is time unit have it enforced)
	float		exp_mod;// 0 = no Exp Ever 1 = normal Exp 2 = float Exp
	float		ammo_mod;// 0 = no Ammo 1 = normal Ammo 2 = float Ammo
	float		hp_mod;// 0 = Invulnerable 1 = normal Damage Recieved 2 = float Damage Recieved

    //Porjectile Stuff
   	float		damage;
   	float		radius;

	void InitEnemy(int i, HealingPoint *_healPoint, Vector _playerPos, float *pPos);
	void AI();
	void Attack();
	void Chase();
	void Damage(Actor *p, int Dmg);
	void Die();
	void Touch(Actor *a);
	void Rebirth();
	void SetNewRanges();
	void CheckHealPoint();
	EnemyAction GetNewAction();	//function to get a new enemy action after death and rebirth
	void DoKamikazeAction();	//different actions the enemies can do
	void DoSurvivorAction();
	void DoGroupieAction();
	void DoConvictAction();
	void DoDodgerAction();
	void DoJumpAction();
	void DoDiveBombAction();
	void MoveTo(float x, float y, float z);
	void SetPlayerPos(Vector pPos);
	void EnemyJump();
};
#endif
